

##########################################################
#####          Fonctions exemple métapop

## Potentiel infectieux
S=function(par,r,O){
  sum(O*par[1]/(2*pi*par[2]^2)*exp(-r/par[2]))
}

## Vraisemblance
llik.dissemination=function(Y,D,theta){
  M=ncol(Y)
  N=nrow(Y)
  out=0
  for(m in 2:M){
	for(i in (1:N)[Y[,m-1]==0]){
	    out=out+dbinom(Y[i,m],1,1-exp(-S(theta[1:2],D[i,],Y[,m-1])),log=TRUE)
  	}
  }
  out
}

## Log-densité de la loi a priori
lprior=function(theta,a){
  dexp(theta[1],1/a[1],log=TRUE)+dexp(theta[2],1/a[2],log=TRUE)
}

## Densité de la loi a priori
prior=function(theta,a){
  cbind(dexp(theta[,1],1/a[1]),dexp(theta[,2],1/a[2]))
}

##########################################################
#####          Fonctions ré-échantillonnage par importance

## Fonction de simulation sous la loi de proposition
rproposal=function(n,par.proposal){
	cbind(rexp(n,1/par.proposal[1]),rexp(n,1/par.proposal[2]))
}

## Fonction logdensité de la loi de proposition
ldproposal=function(theta,par.proposal){
	dexp(theta[,1],1/par.proposal[1],log=TRUE)+dexp(theta[,2],1/par.proposal[2],log=TRUE)
}

## Implémentation du ré-échantillonnage par importance
SIR=function(n,Y,D,a,par.proposal,nprime=n){
	## n: nb de tirage, Y: états sanitaires, D: matrice distance inter-hôtes, a: paramètres de la loi a priori, par.proposal: paramètres de la loi de proposition, nprime: nb de ré-échantillonnage
	## simulation d'un échantillon sous la loi de proposition
	theta=rproposal(n,par.proposal)
	## calcul des termes intervenant dans le calcul des poids
	## logvraisemblance et logprior (seule ligne de la fonction qui dépend du modèle)
	lcible=apply(theta,1,function(th) llik.dissemination(Y,D,th)+lprior(th,a))  
	## logproposal
	lprop=ldproposal(theta,par.proposal)
	## calcul des poids
	lpoids=lcible-lprop
	lpoids=lpoids-max(lpoids)+100 ## ajout d'une cste (indolore) pour des raisons numériques
	poids=exp(lpoids)
	## ré-échantillonnage
 	keep=rmultinom(1,nprime,poids)
  	print(poids)
  	keep.rep=rep(1:n,keep)
  	## output: 
  	## proposed: échantillon initial des paramètres, keep.rep: numéros des paramètres de l'échantillon initial qui sont conservés (avec répétition possibles), posterior: échantillon a posteriori 
  	list(proposed=theta,keep.rep=keep.rep,posterior=theta[keep.rep,])
}

##########################################################
#####          Données métapop

## Lecture des données
source("simul.metapop1.R")
attach(temp)
D=sqrt(outer(x[,1],x[,1],"-")^2+outer(x[,2],x[,2],"-")^2)
M=3

## Graphe positions des hôtes
par(mfrow=c(1,1))
plot(x,pch=19,asp=1,xlab="x (1ère coordonnée)",ylab="x (2ème coordonnée)",
  	cex.lab=1.5,cex.axis=1.5)

## Graphes épidémie observée
par(mfrow=c(1,3))
for(m in 1:M) {
  	plot(x,pch=19,asp=1,xlab="x (1ère coordonnée)",ylab="x (2ème coordonnée)",
	  	cex.lab=1.5,cex.axis=1.5,col=1+(Y[,m]==1))
}

############################################################
#####          Application ré-échantillonnage par importance aux données métapop

## ré-échantillonnage par importance
par.prior=c(2,2)
ech1=SIR(n=10^5,Y=Y,D=D,a=par.prior,par.proposal=c(2,2),nprime=10^4)

## calcul de quantiles à posteriori
apply(ech1$post,2,quantile,c(0.025,0.5,0.975))


par(mfrow=c(1,3))
## Echantillon a posteriori marginal pour theta1
i=1
hist(ech1$post[,i],breaks=16,freq=FALSE,xlab=expression(theta[1]),ylab="Densité",
	main="",ylim=c(0,5.5),col=grey(0.7),border=grey(0.7))
abline(v=theta[i],col=1,lwd=1.5)
lines(seq(0,5,l=200),prior(cbind(seq(0,5,l=200),seq(0,5,l=200)),par.prior)[,i],col=1,
		lwd=1.5,lty="dotted")
box()
## Echantillon a posteriori marginal pour theta2
i=2	
hist(ech1$post[,i],breaks=8,freq=FALSE,xlim=c(0,0.2),xlab=expression(theta[2]),ylab="Densité",
	main="",ylim=c(0,55),col=grey(0.7),border=grey(0.7))
abline(v=theta[i],col=1,lwd=1.5)
lines(seq(0,5,l=200),prior(cbind(seq(0,5,l=200),seq(0,5,l=200)),par.prior)[,i],col=1,
		lwd=1.5,lty="dotted")
box()
## Echantillon a posteriori joint pour theta1 et theta2
draw.nb=NULL
for(i in 1:nrow(ech1$prop)){
	draw.nb=c(draw.nb,sum(ech1$keep.rep==i))
}
plot(ech1$prop[,1],ech1$prop[,2],col=grey(0.7),cex=draw.nb/max(draw.nb)*3,
	xlab=expression(theta[1]),ylab=expression(theta[2]),
	xlim=range(ech1$post[,1]),ylim=range(ech1$post[,2]))
points(theta[1],theta[2],col=1,lwd=2,pch=19,cex=1)

